import java.io.*;
import java.util.*;
import java.util.zip.*;
import au.com.pharos.test.Test;
import au.com.pharos.packing.Packing;
import au.com.pharos.packing.StringPacking;
import au.com.pharos.packing.RawPacking;
import au.com.pharos.gdbm.*;
import IRUtilities.Porter;

public class SearchEngine{
    public static void main(String[] args){
        //Runtime runtime = Runtime.getRuntime();
        //System.out.println ("Free memory :" + runtime.freeMemory() );

    	/* Parameter to define source text file */
    	String strFile="/data2/fcs/Courses/InfoRetrieval/csci6403/W.txt";
    	//String strFile="T.txt";
        String strDBFile="/users/grad/zyu/6403/Repository";
    	try	{
    		/* Create the repository database;*/
      		//Repository r=new Repository(strFile,strDBFile);

      		/* Remove all tags from articles*/
      		//Indexer i=new Indexer();
      		//i.CreateIndex(strDBFile);
      		//create inverted index with weight
      		//new WeightIndexer();

      		//create document similarity matrix database
      		//SimilarityMatrix m=new SimilarityMatrix();
      		//m.Create();

      		//create category
      		new CategoryIndexer();
    	}

    	catch (Exception e) {
    		System.out.println("Error: " + e.getMessage());
    	}
    	finally {
    	}
    }
}

/**
 * A class extracting articles from source text
 * file to Repository database.
 * In repository database, each article is saved as an entry in GDBM file
 * @version	0.1 Feb 19, 2003
 * @author 	Zheyuan Yu
**/

class Repository {
   	/* Buffer is used to improve the performance of I/O,
   	   BUFF_SIZE is the default size of Buffer */
   	static public int BUFF_SIZE = 100000;
	static byte[] buffer;

	public Repository () {
	}

    public Repository(String fileFrom, String fileTo) throws IOException{
    	InputStream in = null;

    	/* A large index would not be feasible to keep in memory at one time,
    	therefore an efficient file utility such as gdbm must be used.*/

	    GdbmFile db = null;


	    try{
	    	new File(fileTo).delete();
	    	buffer= new byte[BUFF_SIZE];
    	    in = new FileInputStream(fileFrom);
		    int intKey=0;
            int intStart; // The start index of the currenct article
            int intLen; // Article length
            String strRem; // the partial article the remained after extraction
	        /* Create the repository database*/
	        db=new GdbmFile(fileTo, GdbmFile.NEWDB);
	        Test.ok(1, db != null);

	    	/* Set the packing strategy*/
	        db.setKeyPacking(new StringPacking());
	        db.setValuePacking(new StringPacking());

	        /* Set the tag of the end of article */
	        String strEndTag="</ARTICLE>";

            strRem="";

            /* Read buffer, extract articles from the buffer
            , compress and insert them into repository database*/
            int amountRead=0;
            while (true) {
                //synchronized (buffer) {
                    amountRead = in.read(buffer);
                    if (amountRead == -1) {
            	        break;
                    }

                    /* merge the two parts of an article which
                    seperated by two buffers*/
                    String strBuffer = strRem+new String(buffer, "ISO-8859-1");
                    int intBuffer=amountRead+strRem.length();

                    //System.out.println ("The buffer has "+Integer.toString(intBuffer)+" bytes, content is :");
                    strRem="";
                    String strArticle="";

                    intStart=0;
                    while (true) {
                    	if (intStart>intBuffer) {
                    		break;
                    	}
                    	/* find the index of the end of the article*/
                        intLen=strBuffer.indexOf(strEndTag,intStart);
                        if (intLen<0) {
                        	int intTmp=strBuffer.length();
                        	strRem=strBuffer.substring(intStart,intTmp);
                        	break;
                        }
                        /* move to the very end, that is just after the end tag,
                        of the current article
                        */
                        intLen+=strEndTag.length();
                        /* extract the current article*/
                        strArticle=strBuffer.substring(intStart,intLen);

                        String strCompressed= IRZIP.compress(strArticle);
                        /* Insert the article to Repository file*/
                        InsertEntry(intKey,strCompressed,db);
                        intKey++;

                        // move the the beginning of next article
                        intStart=intLen+1;
                    }

                //}

            }


        }
	    catch (Exception e) {
    		System.out.println("Error: " + e.getMessage());
	    }

        finally{
            if(in != null)
                in.close();
            if(db !=null)
                db.close();
		    Test.ok(60, true);

        }

    }

    static public void InsertEntry(int intKey, String strItem, GdbmFile db) {

	    try {
	    //System.out.println("Inserted entry: "+Integer.toString(intKey));
        // Store some data in the file
	    db.store(Integer.toString(intKey), strItem);

    	}
	    catch (Exception e) {
    		System.out.println("Error: " + e.getMessage());
	    }
    }
    static public String GetDoc(int intKey,GdbmFile db){
    	String value="";
    	try {
         String strKey=Integer.toString(intKey);
		 value = (String) db.fetch(strKey);
		 value=IRZIP.uncompress(value);
    	}
	    catch (Exception e) {
    		System.out.println("Error: " + e.getMessage());
	    }
	    finally {
	    }
        return (value);
    }
}



/**
 * A class building inverted index and vector file
 * @version	0.1 Feb 19, 2003
 * @author 	Zheyuan Yu
**/

class Indexer {
   	private String strFileFrom;
   	private Vector v;
	public  StopWord sw;
    private TreeMap m;//,lexicon;
    private int intStopWordCount=0;
    static GdbmFile db = null,dbindex=null;
    private int intTermNo=0;
    private HashMap termHashMap;
    private Porter st;

	public Indexer () throws IOException {
	}
	public void CreateIndex(String strDBFile) throws IOException {
		// m is a tree map saving terms in a article in sorted order
		m = new TreeMap();
		termHashMap= new HashMap();
		st = new Porter ();
		v=new Vector();
		//String strFileTo="/users/grad/zyu/6403/vector";
		String strFileTo="/tmp/zyu/vector";
	    new File(strFileTo).delete();
		/* Create the vector file*/
        db=new GdbmFile(strFileTo, GdbmFile.NEWDB);
        Test.ok(1, db != null);
   	    /* Set the packing strategy*/
        db.setKeyPacking(new StringPacking());
        db.setValuePacking(new StringPacking());
        //strFileTo="/users/grad/zyu/6403/index";
        strFileTo="/tmp/zyu/index";
		/* Create the index database file*/
	    new File(strFileTo).delete();
        dbindex=new GdbmFile(strFileTo, GdbmFile.NEWDB);
        Test.ok(2, dbindex != null);
   	    /* Set the packing strategy*/
        dbindex.setKeyPacking(new StringPacking());
        dbindex.setValuePacking(new StringPacking());

		sw=new StopWord();
        BuildTermList(strDBFile);
        strDBFile="/tmp/zyu/termcode";
        BuildTermCode(strDBFile);
        if (db!=null)
            db.close();
        if (dbindex!=null)
            dbindex.close();
	}


	/*Insert Term to Inverted Index File functions*/
	public void InsertTermToInvertedIndex(String strNo, int intDocID) throws IOException {
		try{
			//lexiconEntry lex=null;
			String strIndex="";
			int intDocCount=0; //number of documents under this term
    		strIndex=(String)(dbindex.fetch(strNo));
	    	int d1, d2;
	    	byte[] byteDocCount=new byte[2];
			if (strIndex!=null) {
				//term exist in the inverted index file
				byte[] byteIndex=new byte[strIndex.length()];
				byteIndex=strIndex.getBytes();
				byteDocCount[0]=byteIndex[0];
				byteDocCount[1]=byteIndex[1];
		     	intDocCount=new java.math.BigInteger(byteDocCount).intValue();
				intDocCount++;
     		    /* intDocCount and byteDocCount are  numbers of documents under this term*/
    		    d1=(intDocCount-intDocCount%256)/256;
    		    d2=intDocCount%256;
    		    byteIndex[0]=(byte)(d1);
    		    byteIndex[1]=(byte)(d2);
    		    strIndex=new String(byteIndex);
			}
			else {
    		    //term no exist in the inverted index file
    		    intDocCount++;
    		    /* intDocCount and byteDocCount are  numbers of documents under this term*/
    		    d1=(intDocCount-intDocCount%256)/256;
    		    d2=intDocCount%256;
    		    byteDocCount[0]=(byte)(d1);
    		    byteDocCount[1]=(byte)(d2);
				strIndex=new String(byteDocCount);
			}

   		    d1=(intDocID-intDocID%256)/256;
   		    d2=intDocID%256;
   		    byte[] byteDocID=new byte[2];
   		    byteDocID[0]=(byte)(d1);
   		    byteDocID[1]=(byte)(d2);
   		    String strDocID=new String(byteDocID);
			dbindex.store(strNo,strIndex+strDocID);
		}
		catch (Exception e) {
			System.out.print("Error InsertTermToInVertedIndex "+e.getMessage());
			e.printStackTrace();
		}
    }


	/* insert current sorted term TreeMap to vector file */
	private void InsertToVector(int docid) throws IOException {
		Set mappings = m.entrySet();
		try {
	        String strDocID=Integer.toString(docid);
	        String strEntry="";
	        //iterate through mappings and print content
	        for (Iterator i = mappings.iterator(); i.hasNext();) {
	        	Map.Entry me = (Map.Entry)i.next();
	        	String strTerm = (String)me.getKey(); //word
	        	int intFrequency=((Integer)me.getValue()).intValue();
        		/* intFrequency is number of hits in the article*/
        		byte[] byteFrequency=new byte[1];
        		byteFrequency[0]=(byte)(intFrequency%256);
        		String strFrequency=new String(byteFrequency);
        		/* Forward index file structure
        		docid, word, frequency*/
        		int intNo=((Integer)termHashMap.get(strTerm)).intValue();
        		byte[] byteNo=new byte[3];
        		int d1=(intNo-intNo%65536)/65536;
        		int dtmp=intNo%65536;
  		        int d2=(dtmp-dtmp%256)/256;
   		        int d3= dtmp%256;
     		    byteNo[0]=(byte)(d1);
   		        byteNo[1]=(byte)(d2);
   		        byteNo[2]=(byte)(d3);
        		String strNo=new String (byteNo);
        		strEntry+=strNo+strFrequency;
        		/* insert this term to inverted TreeMap*/
        		InsertTermToInvertedIndex(strNo, docid);
//        		System.out.println("Inserting "+strKey+" docid "+docid+" strHits " +strHits+" Number of hits "+intNHits+ " length "+strHits.length());
//        		System.in.read();
            }
           // strEntry=IRZIP.compress(strEntry);
            db.store(strDocID, strEntry);
            m.clear();
  	    }
        catch (Exception e) {
        	System.err.println ("Error writing to vector file");
        }
	}

	private void InsertOrAppendToMapOfTermList(String strTerm) throws IOException {
		int intFrequency=0;
		if (m.containsKey(strTerm)) {
			intFrequency=((Integer)m.get(strTerm)).intValue();
		}
		intFrequency++;
		m.put(strTerm,new Integer(intFrequency));
		if (!(termHashMap.containsKey(strTerm))) {
			termHashMap.put(strTerm,new Integer(intTermNo));
			intTermNo++;
		}
	}

 	public String Stemming(String strTerm){
		return st.stripAffixes(strTerm);
	}

	public void BuildTermListProcessing (int docid, String s) throws IOException {
		if ((docid%200)==0)
		    System.out.println("processing doc "+docid);
		/* source byte array is used to save article*/
		byte [] source=s.getBytes("ISO-8859-1");
    	int intLen=source.length;
		//true: tag   false:plain text
		boolean flag=false;
		int i=0, j=0, intStart=0, intEnd=0, intOffset=0;
		//process every characters in the article
		for (i=0; i<intLen; i++) {

            if ((source[i]==' ')||(source[i]==10)||(source[i]==13)) //space, LR, CR
                continue;

			if (source[i]=='<') { //start of tag
				flag=true;
				continue;
			}

			if (source[i]=='>') { //end of tag
		        flag=false;
		        continue;
		    }

			if (flag==false){  //term
				// find the start and end of a term
				intStart=i;
				intEnd=i;
			    while ((source[i]!=' ')&&(source[i]!=10)&&(source[i]!=13)&&(source[i]!='<')&&(i<intLen)) {
			        i++;
			        intEnd++;
			    }
			    i--;
			    int intTermIndex=0;
			    //, intCurrentWordLength=0;

			    byte[] term=new byte [intEnd-intStart];
			    for (int k=intStart;k<intEnd;k++) {
	    			// convert to lower case
	    			if ((source[k]>='A')&&(source[k]<='Z')) {
	    				source[k]+=32;
	    			}
    				//filter out the control characters
    				if (
    					!(
    						((source[k]<='9')&&(source[k]>='0'))
    						||((source[k]<='Z')&&(source[k]>='A'))
    						||((source[k]<='z')&&(source[k]>='a'))
/*    						||(source[k]==' ')
    						||(source[k]=='<')
    						||(source[k]=='>')
    						*/
				        )
				    )
				    continue;
	    			term[intTermIndex]=source[k];
	    			intTermIndex++;
	    		}

	    		// create a term array which contain only characters of term.

				byte[] cterm=new byte[intTermIndex];
				int n=0;
				boolean blnContainNumber=false;
				for (int m=0;m<intTermIndex;m++) {
					if ((term[m]==' ')||(term[m]==10))
					    continue;
					if ((term[m]<='9')&&(term[m]>='0'))
					    blnContainNumber=true;
					cterm[n++]=term[m];

			    }
			    if (n==0) //empty term
			        continue;
			    if (blnContainNumber)  //term contain number is filtred.
			        continue;

				if (sw.IsStopWord(cterm)) {
				    intStopWordCount++;
				}
				else {
					String strTerm=new String(cterm,"ISO-8859-1");
			        strTerm=Stemming(strTerm);
					InsertOrAppendToMapOfTermList(strTerm);
			    }
			}
		}
    }

    public void BuildTermCode(String strFile) throws IOException {
    	try {
	    	db = new GdbmFile(strFile, GdbmFile.NEWDB);
	    	db.setKeyPacking(new StringPacking());
	    	db.setValuePacking(new StringPacking());
    		Set mappings = termHashMap.entrySet();
            //iterate through term code mappings
	        for (Iterator i = mappings.iterator(); i.hasNext();) {
	        	Map.Entry me = (Map.Entry)i.next();
	        	String strTerm = (String)me.getKey(); //word
	        	int intNo=((Integer)me.getValue()).intValue();
        		byte[] byteNo=new byte[3];
        		int d1=(intNo-intNo%65536)/65536;
        		int dtmp=intNo%65536;
  		        int d2=(dtmp-dtmp%256)/256;
   		        int d3= dtmp%256;
     		    byteNo[0]=(byte)(d1);
   		        byteNo[1]=(byte)(d2);
   		        byteNo[2]=(byte)(d3);
        		String strNo=new String (byteNo);
        		db.store(strTerm,strNo);
        		System.out.println(strTerm);
	        }
	        db.close();
    	}
    	catch (Exception e) {
    		System.out.println("Error BuildTermCode "+e.getMessage());
    	}
    }

    public void BuildTermList(String strFile) throws IOException {
	    GdbmFile db = null;
	    try {
	    	// Open a database read-only
	    	db = new GdbmFile(strFile, GdbmFile.READER);
	    	db.setKeyPacking(new StringPacking());
	    	db.setValuePacking(new StringPacking());
	    	int intDocID = 0;
	    	String key;
	    	String strArticle="";
	    	Enumeration keys = db.keys();
	    	while ( keys.hasMoreElements() ) {
	    		keys.nextElement();
	    		strArticle=Repository.GetDoc(intDocID,db);
   				BuildTermListProcessing(intDocID, strArticle);
   				/* insert the terms of currect article to forward vector*/
   				InsertToVector(intDocID);
   				int intReorganize=0;
   				/*
   				if (intDocID<1000) {
   				   intReorganize=1000;
   				 }
   				 else {
   				 	if (intDocID<2000){
   				 		intReorganize=500;
   				 	}
   				 	else {
   				 		if (intDocID<3000){
   				 			intReorganize=380;
   				 		}
   				 		else {
   				 			if (intDocID<4000){
   				 				intReorganize=250;
   				 			}
   				 			else {
   				 				if (intDocID<5000){
   				 					intReorganize=125;
   				 				}
   				 				else {
   				 					if (intDocID<7000){
   				 						intReorganize=60;
   				 					}
   				 					else {
   				 						intReorganize=30;
   				 					}
   				 				}
   				 			}
   				 		}
   				 	}

   				 }

		        if ((intDocID%intReorganize)==0) {
                    dbindex.reorganize();
                }*/
                intDocID++;
   			}
            dbindex.reorganize();
       //     CompressIndex();

   			System.out.println("Total number of stopwords "+intStopWordCount);
            if(db !=null)
                db.close();
            if(dbindex!=null)
                dbindex.close();
   		}
	    catch (Exception e) {
	    	System.out.println("Error to remove tags from string\n"+e.getMessage()+"\n");
	    	e.printStackTrace();
	    }
    }
}




/**
 * A class building inverted index with weight
 * @version	0.1 Feb 19, 2003
 * @author 	Zheyuan Yu
**/

class WeightIndexer {
	public WeightIndexer (){
		try {
   	        //String strWeightFile="/users/grad/zyu/6403/weight";
   	        String strWeightFile="/tmp/zyu/weight";
   	        String strIndexFile="/tmp/zyu/index";
   	        String strVectorFile="/tmp/zyu/vector";
   	        //String strRepositoryFile="/users/grad/zyu/6403/Repository";
	    	new File(strWeightFile).delete();
	    	System.out.println("ok");


	    	System.out.println("ok");
            GdbmFile indexDatabase=new GdbmFile(strIndexFile, GdbmFile.READER);
     	    GdbmFile vectorDatabase=new GdbmFile(strVectorFile,GdbmFile.READER);
     	    //GdbmFile repositoryDatabase=new GdbmFile(strRepositoryFile,GdbmFile.READER);
            //Test.ok(3, repositoryDatabase!= null);
     	    /* create term weight database*/
	    	new File(strWeightFile).delete();
     	    GdbmFile weightDatabase=new GdbmFile(strWeightFile, GdbmFile.NEWDB);
   	        /* Set the packing strategy*/
            indexDatabase.setKeyPacking(new StringPacking());
            indexDatabase.setValuePacking(new StringPacking());
            vectorDatabase.setKeyPacking(new StringPacking());
            vectorDatabase.setValuePacking(new StringPacking());
            //repositoryDatabase.setKeyPacking(new StringPacking());
            //repositoryDatabase.setValuePacking(new StringPacking());
	        weightDatabase.setKeyPacking(new StringPacking());
	        weightDatabase.setValuePacking(new StringPacking());
			//Search s=new Search();
	        HashMap idfHashMap=new HashMap();
        	//IRZIP zip=new IRZIP();

    	    int intTotalDoc=vectorDatabase.size();
    	    // read term from index database one by one
    	    // calculate TF and IDF=N/DF for each term
    	    Enumeration keys = indexDatabase.keys();
    	    while ( keys.hasMoreElements() ) {
    	    	String strTermNo = (String) keys.nextElement();
    	    	String strHitSet="";
     	        //Term IDF
                strHitSet = (String) indexDatabase.fetch(strTermNo);
       		    byte[] byteDocCount=strHitSet.substring(0,2).getBytes();
	            int intDocCount=new java.math.BigInteger(byteDocCount).intValue();
	            double dblIDF=Math.log((double)intTotalDoc/(double)intDocCount)/Math.log(2);
	            idfHashMap.put(strTermNo,new Double(dblIDF));
    	    }
    	    double dblMax=0.0;
    	    keys=vectorDatabase.keys();
    	    while (keys.hasMoreElements()) {
    	    	String strDocID = (String) keys.nextElement();
    	    	int intDocID=new Integer(strDocID).intValue();
    	    	System.out.println("Doc "+strDocID);
            	String strVector="";
    		    strVector =(String) vectorDatabase.fetch(strDocID);
    	        int intFrequency=0;
	    	    int intMaxHit=0;
    	    	double dblSumWeightSquare=0.0;
                byte[] byteVector=strVector.getBytes();
                int intLen=byteVector.length;
                int intLoop=(intLen-intLen%4)/4;
    	        TreeMap weightHashMap = new TreeMap();

    	        for (int i=0;i<intLoop;i++) {
	                byte[] byteTerm=new byte[3];
	                byteTerm[0]=byteVector[4*i];
	                byteTerm[1]=byteVector[4*i+1];
	                byteTerm[2]=byteVector[4*i+2];
	                String strTermNo=new String(byteTerm);
	                //intFrequency=(int)(byteVector[4*i+3]);
	                byte[] byteFrequency=new byte[1];
	                byteFrequency[0]=byteVector[4*i+3];
                    intFrequency=new java.math.BigInteger(byteFrequency).intValue();
     	     	    double dblIDF=((Double)idfHashMap.get(strTermNo)).doubleValue();
     	     	    double dblWeight=dblIDF*(double)intFrequency;
     	     	    dblSumWeightSquare+=dblWeight*dblWeight;
     	     	    weightHashMap.put(strTermNo,new Double (dblWeight));
     	     	    // get the max number of hit
     	     	    if (intMaxHit<intFrequency)
        	     	    intMaxHit=intFrequency;
     	        }
           	    Set mappings = weightHashMap.entrySet();
    	        //iterate through weigth hash map
    	        String strWeight="";
    	        for (Iterator i = mappings.iterator(); i.hasNext();) {
    	        	Map.Entry me = (Map.Entry)i.next();
    	        	double dblWeight = ((Double)me.getValue()).doubleValue();
    	        	String strTermNo=(String)me.getKey();
    	        	//dblWeight=dblWeight/(double)intMaxHit;
    	        	dblWeight=dblWeight/java.lang.Math.sqrt(dblSumWeightSquare);
    	        	if (dblWeight>dblMax) {
    	        	    System.out.println(dblWeight);
    	        	    dblMax=dblWeight;
    	        	}
    	        	//dblWeight=dblWeight;
    	        	java.math.BigDecimal Big=new java.math.BigDecimal(dblWeight*10000000.0);
    	        	int intWeight=Big.intValue();
        	        int d1=(intWeight-intWeight%65536)/65536;
  		            int dtmp=intWeight%65536;
  		            int d2=(dtmp-dtmp%256)/256;
   		            int d3= dtmp%256;
   		            byte[] byteWeight=new byte[3];
   		            byteWeight[0]=(byte)(d1);
   		            byteWeight[1]=(byte)(d2);
   		            byteWeight[2]=(byte)(d3);
   	            	strWeight+=strTermNo+(new String(byteWeight));
          	    }
         	    weightDatabase.store(strDocID,strWeight);
    	    }
    	    System.out.println("dblMax "+dblMax);

   	    }
   	    catch (Exception e) {
   	    	System.out.println("Error WeightIndex "+e.getMessage());
   	    	e.printStackTrace();

   	    }

	}

}



/**
 * A class building category index file
 * @version	0.1 Feb 19, 2003
 * @author 	Zheyuan Yu
**/

class CategoryIndexer {
	public CategoryIndexer (){
		try {
			Search s=new Search();
   	        String strCategoryFile="/users/grad/zyu/6403/category";
   	        String strIndexFile="/users/grad/zyu/6403/index";
   	        String strVectorFile="/users/grad/zyu/6403/vector";
   	        String strRepositoryFile="/users/grad/zyu/6403/Repository";
   	        String strTermFile="/users/grad/zyu/6403/term";
	    	new File(strCategoryFile).delete();
	    	new File(strTermFile).delete();
            GdbmFile indexDatabase=new GdbmFile(strIndexFile, GdbmFile.READER);
            Test.ok(1, indexDatabase!= null);
     	    GdbmFile vectorDatabase=new GdbmFile(strVectorFile,GdbmFile.READER);
            Test.ok(2, vectorDatabase!= null);
     	    GdbmFile repositoryDatabase=new GdbmFile(strRepositoryFile,GdbmFile.READER);
            Test.ok(3, repositoryDatabase!= null);
     	    /* create term weight database*/
     	    GdbmFile categoryDatabase=new GdbmFile(strCategoryFile, GdbmFile.NEWDB);
            Test.ok(4, categoryDatabase!= null);
     	    GdbmFile termDatabase=new GdbmFile(strTermFile, GdbmFile.NEWDB);
            Test.ok(5, termDatabase!= null);


   	        /* Set the packing strategy*/
            indexDatabase.setKeyPacking(new StringPacking());
            indexDatabase.setValuePacking(new StringPacking());
            vectorDatabase.setKeyPacking(new StringPacking());
            vectorDatabase.setValuePacking(new StringPacking());
            repositoryDatabase.setKeyPacking(new StringPacking());
            repositoryDatabase.setValuePacking(new StringPacking());
	        categoryDatabase.setKeyPacking(new StringPacking());
	        categoryDatabase.setValuePacking(new StringPacking());
	        termDatabase.setKeyPacking(new StringPacking());
	        termDatabase.setValuePacking(new StringPacking());


//	        HashMap idfHashMap=new HashMap();
        	//IRZIP zip=new IRZIP();

    	    int intTotalDoc=repositoryDatabase.size();
    	    // read term from index database one by one
    	    // calculate TF and IDF=N/DF for each term
    	    Enumeration keys = indexDatabase.keys();
/*    	    while ( keys.hasMoreElements() ) {
    	    	String strTerm = (String) keys.nextElement();
//    	    	System.out.println(strTerm);
    	    	String strHitSet="";
     	        //Term IDF
                strHitSet = (String) indexDatabase.fetch(strTerm);
       		    byte[] byteDocCount=strHitSet.substring(0,2).getBytes();
	            int intDocCount=new java.math.BigInteger(byteDocCount).intValue();
	            double dblIDF=Math.log((double)intTotalDoc/(double)intDocCount)/Math.log(2);
	            idfHashMap.put(strTerm,new Double(dblIDF));
    	    }*/
    	    TreeMap categoryTreeMap= new TreeMap();
  	        TreeMap termHashMap = new TreeMap();

int intCount=0;

    	    keys=vectorDatabase.keys();
    	    while (keys.hasMoreElements()) {
    	    	if (intCount>=5000) break;

    	    	String strDocID = (String) keys.nextElement();
    	    	int intDocID=new Integer(strDocID).intValue();
    	    	String strDoc=s.getDoc(intDocID);
    	    	String strCategory=s.getCategory(strDoc);
    	    	System.out.println("Doc "+strDocID+" category "+strCategory);
    	    	intCount++;
    	    	System.out.println(intCount);
    	    	String strDocIDList="";
    	    	if (categoryTreeMap.containsKey(strCategory)) {
    	    		strDocIDList=(String)categoryTreeMap.get(strCategory);
    	    	}
    	    	int d1=(intDocID-intDocID%256)/256;
    	    	int d2=intDocID%256;
    	    	byte[] byteDocID=new byte[2];
    		    byteDocID[0]=(byte)(d1);
    		    byteDocID[1]=(byte)(d2);
    	    	strDocIDList+=new String(byteDocID);
    	    	categoryTreeMap.put(strCategory,strDocIDList);
            	String strVector="";
    		    strVector =(String)vectorDatabase.fetch(strDocID);
    	        int intFrequency=0;
    	    	double dblSumWeightSquare=0.0;
                byte[] byteVector=strVector.getBytes();
                int intLen=byteVector.length;
	            int intLoop=(intLen-intLen%6)/6;
    	        byte[] byteNo= new byte[3];
        	    byte[] byteFrequency= new byte[3];
    	        for (int i=0;i<intLoop;i++) {

    	        	byteNo[0]=byteVector[i*4];
      	    	    byteNo[1]=byteVector[i*4+1];
      		        byteNo[2]=byteVector[i*4+2];
        	        String strNo=new String(byteNo);
        	        byteFrequency[0]=0;
        	        byteFrequency[1]=0;
	                byteFrequency[2]=byteVector[i*4+3];
	                String strFrequency=new String(byteFrequency);
	     	        intFrequency=new java.math.BigInteger(byteFrequency).intValue();
	     	        String strTermList="";
	     	        if (termHashMap.containsKey(strCategory))
	     	            strTermList=(String)termHashMap.get(strCategory);
	     	        byte[] byteTermList=strTermList.getBytes();
	     	        HashMap h;
	     	        strTermList=getTermInCategory(byteTermList,byteNo,intFrequency);
	     	        //strTermList=setTermListInCategory(h,byteNo);
	     	        /*
	     	        if (intIndex>=0) {
	     	        	//this term is in the category term hashmap, update
	     	        	byteFrequency[0]=byteTermList[intIndex+3];
	     	        	byteFrequency[1]=byteTermList[intIndex+1+3];
	     	        	byteFrequency[2]=byteTermList[intIndex+2+3];
	     	            int intOriginalFrequency=new java.math.BigInteger(byteFrequency).intValue();
	     	            intFrequency+=intOriginalFrequency;
	     	            d1=(intFrequency-intFrequency%65536)/65536;
        		        int dtmp=intFrequency%65536;
  		                d2=(dtmp-dtmp%256)/256;
   		                int d3= dtmp%256;
     		            byteTermList[intIndex+3]=(byte)(d1);
   		                byteTermList[intIndex+1+3]=(byte)(d2);
   		                byteTermList[intIndex+2+3]=(byte)(d3);
	     	        	strTermList=new String(byteTermList);
	     	        }
	     	        else {
	     	        	//this term not in the category term hashmap, append
	     	        	strTermList+=strNo+strFrequency;
	     	        }*/
     	     	    termHashMap.put(strCategory,strTermList);
     	        }
    	    }

           	Set mappings = termHashMap.entrySet();
    	    //iterate through weigth hash map
    	    String strTermList="";
    	    for (Iterator i = mappings.iterator(); i.hasNext();) {
    	    	Map.Entry me = (Map.Entry)i.next();
    	    	strTermList = (String)me.getValue();
    	    	String strCategory=(String)me.getKey();
                termDatabase.store(strCategory,strTermList);
         	}

           	mappings = categoryTreeMap.entrySet();
    	    //iterate through category hash map
    	    for (Iterator i = mappings.iterator(); i.hasNext();) {
    	    	Map.Entry me = (Map.Entry)i.next();
   	            String strCategory=(String)me.getKey();
   	        	String strDocIDList=(String)me.getValue();
   	        	System.out.println(strCategory);
   	        	categoryDatabase.store(strCategory,strDocIDList);
   	        }
   	    }
   	    catch (Exception e) {
   	    	System.out.println("Error CategoryIndexer "+e.getMessage());
   	    	e.printStackTrace();

   	    }

	}
	public String getTermInCategory (byte[] byteTermList, byte[] byteTermNo, int intTermFrequency) {
		String strReturn="";
		HashMap h=new HashMap();
		int intLen=byteTermList.length;
		int intLoop=(intLen-intLen%6)/6;
		String strTermNo=new String(byteTermNo);
		for (int i=0;i<intLoop;i++) {
            byte[] byteNo= new byte[3];
          	byteNo[0]=byteTermList[i*6];
      	    byteNo[1]=byteTermList[i*6+1];
      		byteNo[2]=byteTermList[i*6+2];
        	String strNo=new String(byteNo);
            if (strNo.compareTo(strTermNo)==0){
                byte[] byteFrequency=new byte[3];
                byteFrequency[0]=byteTermList[i*6+3];
                byteFrequency[1]=byteTermList[i*6+4];
                byteFrequency[2]=byteTermList[i*6+5];
                int intFrequency=new java.math.BigInteger(byteFrequency).intValue();
                intFrequency+=intTermFrequency;
    	        int d1=(intFrequency-intFrequency%65536)/65536;
                int dtmp=intFrequency%65536;
  		        int d2=(dtmp-dtmp%256)/256;
   		        int d3= dtmp%256;
     	        byteTermList[i*6+3]=(byte)(d1);
     	        byteTermList[i*6+4]=(byte)(d2);
   		        byteTermList[i*6+5]=(byte)(d3);
   		       // System.out.println("found");
   		        return (new String(byteTermList));
   		    }
        }
        int intFrequency=intTermFrequency;
    	int d1=(intFrequency-intFrequency%65536)/65536;
        int dtmp=intFrequency%65536;
  		int d2=(dtmp-dtmp%256)/256;
   		int d3= dtmp%256;
   		byte[] byteFrequency=new byte[3];
     	byteFrequency[0]=(byte)(d1);
     	byteFrequency[1]=(byte)(d2);
   		byteFrequency[2]=(byte)(d3);
   		//System.out.println("not found");

        return ((new String(byteTermList))+strTermNo+(new String(byteFrequency)));
        /*



                h.put(strNo,new Integer(intFrequency));
		}
		if (h.containsKey(strTermNo)) {
			int intOldFrequency=((Integer)h.get(strTermNo)).intValue();
			h.put(strTermNo,new Integer(intOldFrequency+intTermFrequency));
			//System.out.println(intTermFrequency);
		}
		else {
			h.put(strTermNo, new Integer(intTermFrequency));
		}
   	    Set mappings = h.entrySet();
        //iterate through hash map
        for (Iterator i = mappings.iterator(); i.hasNext();) {
    	     Map.Entry me = (Map.Entry)i.next();
        	 strTermNo=(String)me.getKey();
         	 int intFrequency=((Integer)me.getValue()).intValue();
	         int d1=(intFrequency-intFrequency%65536)/65536;
             int dtmp=intFrequency%65536;
  		     int d2=(dtmp-dtmp%256)/256;
   		     int d3= dtmp%256;
     		 byte[] byteTerm=new byte[3];
     		 byteTerm[0]=(byte)(d1);
     		 byteTerm[1]=(byte)(d2);
   		     byteTerm[2]=(byte)(d3);
	     	 strReturn+=strTermNo+new String(byteTerm);
	    }
		return strReturn;
*/


	/*	String strTermNo=new String (byteTermNo);
        int intLen=byteTermList.length;
        int intLoop=(intLen-intLen%6)/6;
        int low=0, high=intLoop-1, mid=0;
		try {
            while (low<=high){
        	    mid=low+high;
        	    mid=(mid-mid%2)/2; //mid index in the sublist
            	byte[] byteNo= new byte[3];
          		byteNo[0]=byteTermList[mid*6];
      	    	byteNo[1]=byteTermList[mid*6+1];
      		    byteNo[2]=byteTermList[mid*6+2];
        	    String strNo=new String(byteNo);
                int intCompare=strNo.compareTo(strTermNo);
            	if (intCompare==0){
            		// have a match.
                    //byte[] byteFrequency=new byte[3];
                    //byteFrequency[0]=byteTermList[mid*6+3];
                    //byteFrequency[1]=byteTermList[mid*6+4];
                    //byteFrequency[2]=byteTermList[mid*6+5];
                    //int intFrequency=new java.math.BigInteger(byteFrequency).intValue();
                    //System.out.println("Find term");
                    return mid*6;
		        }
    	        if (intCompare>0)
                    high=mid-1; //go to upper sublist
                else
                    low=mid+1;
		    }
            return -1;
        }
        catch (Exception e) {
    	    System.out.println("Error getIndexOfTermInCategory "+e.getMessage());
        }
        return -1;
        */



   	}



}




/**
 * A class building similarity matrix
 * @version	0.1 March 26, 2003
 * @author 	Zheyuan Yu
**/

class SimilarityMatrix {
	public SimilarityMatrix (){
	}
	public void Create(){
		try {
			Search s=new Search();
   	        //String strWeightFile="/users/grad/zyu/6403/weight";
   	        String strWeightFile="/tmp/zyu/weight";
   	        String strSimilarityFile="/users/grad/zyu/6403/similarity";
   	        String strRepositoryFile="/users/grad/zyu/6403/Repository";

	    	new File(strSimilarityFile).delete();
     	    GdbmFile weightDatabase=new GdbmFile(strWeightFile,GdbmFile.READER);
            Test.ok(1, weightDatabase!= null);
     	    // create document similarity database
     	    GdbmFile similarityDatabase=new GdbmFile(strSimilarityFile, GdbmFile.NEWDB);
            Test.ok(2, similarityDatabase!= null);

   	        // Set the packing strategy
            similarityDatabase.setKeyPacking(new StringPacking());
            similarityDatabase.setValuePacking(new StringPacking());
	        weightDatabase.setKeyPacking(new StringPacking());
	        weightDatabase.setValuePacking(new StringPacking());

    	    int intTotalDoc=weightDatabase.size();
    	    for (int i=1;i<intTotalDoc;i++) {

    	    	for (int j=0;j<=i-1;j++) {
    	    		double dblSimilarity=s.getDocSimilarity(i,j);
    	    		System.out.println(" ("+i+","+j+") "+dblSimilarity);
    	    	}
    	    }


   	    }
   	    catch (Exception e) {
   	    	System.out.println("Error SimilarityWeight "+e.getMessage());
   	    	e.printStackTrace();

   	    }

	}
}
